#include "framecpp/config.h"

#include <memory>

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"

#include "framecpp/Version3/FrMsg.hh"
#include "framecpp/Version3/FrSE.hh"
#include "framecpp/Version3/FrSH.hh"

#include "framecpp/Version3/PTR_STRUCT.hh"

using namespace FrameCPP::Version_3;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;


//=======================================================================
// Static
//=======================================================================
static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_MSG;

//=======================================================================
// FrMsg::fr_event_data_type
//=======================================================================

bool FrMsg::fr_msg_data_type::
operator==( const fr_msg_data_type& RHS ) const
{
#define CMP__(X) ( X == RHS.X )

  return ( ( &RHS == this )
	   || ( CMP__( alarm )
		&&  CMP__( message )
		&&  CMP__( severity )
	   ) )
    ;
#undef CMP__
}

//=======================================================================
// FrMsg
//=======================================================================
FrMsg::
FrMsg( )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
}

FrMsg::
FrMsg( const FrMsg& Source )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.alarm = Source.m_data.alarm;
  m_data.message = Source.m_data.message;
  m_data.severity = Source.m_data.severity;
}

FrMsg::
FrMsg( const std::string& Alarm,
       const std::string& Message,
       INT_4U Severity )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.alarm = Alarm;
  m_data.message = Message;
  m_data.severity = Severity;
}

FrMsg::
FrMsg( IStream& Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  Stream >> m_data.alarm
	 >> m_data.message
	 >> m_data.severity
    ;
  Stream.Next( this );
}

const std::string& FrMsg::
GetAlarm( ) const
{
   return m_data.alarm;
}


FrameCPP::cmn_streamsize_type FrMsg::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.alarm.Bytes( )
    + m_data.message.Bytes( )
    + sizeof( m_data.severity )
    + Stream.PtrStructBytes( )	// next
    ;
}

FrMsg* FrMsg::
Create( IStream& Stream ) const
{
  return new FrMsg( Stream );
}

std::auto_ptr< FrameSpec::Object > FrMsg::
Promote( INT_2U Source,
	 FrameSpec::Object* Obj, IStream* Stream )
{
  return Common::PromoteObject( DATA_FORMAT_VERSION, Source, Obj );
}

const char* const FrMsg::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrMsg::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {

    ret( FrSH( FrMsg::StructName( ), s_object_id,
	       "Msg Data Structure" ) );

    ret( FrSE( "alarm", "STRING",
	       "" ) );
    ret( FrSE( "message", "STRING",
	       "" ) );
    ret( FrSE( "severity", "INT_4U",
	       "" ) );

    ret( FrSE( "next", PTR_STRUCT::Desc( FrMsg::StructName( ) ),
	       "" ) );
  }

  return &ret;
}

void FrMsg::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  va_end( vl );
}

void FrMsg::
Write( OStream& Stream ) const
{
  Stream << m_data.alarm
	 << m_data.message
	 << m_data.severity
    ;
  WriteNext( Stream );
}

FrMsg& FrMsg::
Merge( const FrMsg& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}

bool FrMsg::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

std::auto_ptr< FrameSpec::Object > FrMsg::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrMsg* >( this ) );
  }
  throw
    Unimplemented( "Object* FrMsg::demote( INT_2U Target, Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrMsg::
promote( INT_2U Source, Object* Obj, IStream* Stream ) const
{
  std::auto_ptr< FrameSpec::Object >	retval;
  if ( Source >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >( Obj );
  }
  throw
    Unimplemented( "Object* FrMsg::promote( INT_2U Source, Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}
